sets s      suppliers            /S1,S2/
     n      nodes                /PN,LN,RN,CN/
     z      activities           /P1,P2,PC,PL,LR,RC,I1,X1,W1,D1,D2/
     t      time periods         /T1*T2/
     zf(z)  flow-based activities - in contrast to volume based (working gas)
     zp(z)  prod activities
     za(z)  arc-flow activities - pipelines & LNG chain & stor inj and extract
     zl(z)  liquefaction
     zr(z)  regasification
     zd(z)  demand activities
     znd(z) not demand activities
     zi(z)  injection
     zx(z)  extraction
     zw(z)  working gas
alias (s,s1),(n,n1),(t,t1);
table dat(z,*)
    c  d e cap l exp lt S1 S2 prd arc PN LN RN CN g W1 vol liq reg inj ext res
P1  1      10            1     1      +1          0                        99
P2  0 .2    6               1  1      +1                                   99
PC 1.8     10 .03        1         1  -1       +1
PL  0    1  0 .10 99  1     1      1  -1 +1                 1
LR  1      10 .01           1      1     -1 +1
RC  1      10 .02           1      1        -1 +1               1
I1  1       3 .02        1                     -1   1               1
X1  1       4            1                     +1   1                   1
W1          5            1                              1
D1                       1  1                  -1
D2                       1  1                  -1
+   a b t
D1 10 1 10
D2 10 1 10
;
parameter cv(s,z) market power /S1.D1 1/
          g(z,t,t1) depreciation
          r(t) discount factor;
r(t)=power(0.9,ord(t)-1);

*flow-based activities
zf(z)$(NOT dat(z,'vol'))=1;
*arc-based activities
za(z)=dat(z,'arc');
zl(z)=dat(z,'liq');
zr(z)=dat(z,'reg');
*production activities
zp(z)=dat(z,'prd');
*storage activities
zi(z)=dat(z,'inj');
zx(z)=dat(z,'ext');
zw(z)$(sum(zi,dat(zi,z)) AND sum(zx,dat(zx,z)))=1;
*demand activities
zd(z)$dat(z,'b')=1;
*not demand activities
znd(z)=NOT(zd(z));

*Correct incoming flows for losses
dat(zf,n)$(dat(zf,n)>0)=dat(zf,n)*(1-dat(zf,'l'));

*depreciation - currently only applies to new expansions
g(znd,t,t1)$(ord(t1)>ord(t)+dat(znd,'lt'))=dat(znd,'g');

*minimum lead time one period
dat(znd,'lt')=max(1,dat(znd,'lt'));

variable           ZO       objective
positive variables Q(s,z,t) quantities,
                   D(z,t)   capacity expansion;
equations          obj, cap, mb mass bal, stor inventory, wg working gas, res;

obj..
  ZO =E= sum(t,r(t)*
     [  sum{(s,zd),(dat(zd,'a')+dat(zd,'t')*(ord(t)-1)-dat(zd,'b')*
                                                  sum(s1,Q(s1,zd,t)))*Q(s,zd,t)}
      + sum{   zd,  dat(zd,'b')*sum(s,Q(s,zd,t))*sum(s,Q(s,zd,t))}/2
      - sum{(s,zd), cv(s,zd)*dat(zd,'b')*Q(s,zd,t)*Q(s,zd,t)}/2
      - sum{ znd,  (dat(znd,'c')+dat(znd,'d')*sum(s,Q(s,znd,t)))*
                                                             sum(s,Q(s,znd,t))}
      - sum{ znd,  (dat(znd,'e')*D(znd,t))}
     ]);
*Restrict access to activities
Q.fx(s,znd,t)$(NOT dat(znd,s))=0;

cap(znd,t)..  sum(s,Q(s,znd,t)) =L= dat(znd,'cap')*(1-g(znd,'T1',t)) +
              sum(t1$(ord(t1)<=ord(t)-dat(znd,'lt')),(1-g(znd,t1,t))*D(znd,t1));

mb(s,n,t)..   sum(zf,dat(zf,n)*Q(s,zf,t)) =E= 0;

res(zp)..    sum((s,t),Q(s,zp,t)) =L= dat(zp,'res');

stor(s,zw,t).. sum((t1,zx)$(ord(t1)<=ord(t) AND dat(zx,zw)=1),Q(s,zx,t1)) =L=
               sum((t1,zi)$(ord(t1)< ord(t) AND dat(zi,zw)=1),Q(s,zi,t1)*
                                                               (1-dat(zi,'l')));
wg(zw,t)..
   sum((s,t1,zi)$(ord(t1)<=ord(t) AND dat(zi,zw)=1),Q(s,zi,t1)*(1-dat(zi,'l')))
 - sum((s,t1,zx)$(ord(t1)< ord(t) AND dat(zx,zw)=1),Q(s,zx,t1)) =L=
 dat(zw,'cap') + sum(t1$(ord(t1)<=ord(t)-dat(zw,'lt')),(1-g(zw,t1,t))*D(zw,t1));

*restrict expansion level
D.up(znd,t)=dat(znd,'exp');

model NG /all/;
option qcp=cplex;
solve NG max ZO using qcp;

parameter price, rep(t,*,*,*,*) "extended mass balance report, monetary results are not discounted ";

price(t,zd)=(dat(zd,'a')+dat(zd,'t')*(ord(t)-1)-dat(zd,'b')*sum(s,Q.l(s,zd,t)));

*eliminate insignificant values
Q.l(s,z,t)=round(Q.l(s,z,t),5);
D.l(znd,t)=round(D.l(znd,t),5);

rep(t,s,n,'+','prod')=     sum(zp(z)$(dat(z,n)>0),         Q.l(s,z,t));
rep(t,s,n,'+','flow')=     sum(za$(dat(za,n)>0),dat(za,n)* Q.l(s,za,t));
rep(t,s,n,'-','flow')=     sum(za$(dat(za,n)<0),           Q.l(s,za,t));
rep(t,s,n,'+','(LNG)')=    sum(zl(z)$(dat(z,n)>0),dat(z,n)*Q.l(s,z,t));
rep(t,s,n,'-','(LNG)')=    sum(zr(z)$(dat(z,n)<0),         Q.l(s,z,t));
rep(t,s,n,'+','Stor')=     sum(zx(z)$(dat(z,n)>0),dat(z,n)*Q.l(s,z,t));
rep(t,s,n,'-','Stor')=     sum(zi(z)$(dat(z,n)<0),         Q.l(s,z,t));
rep(t,s,n,'-',zd)$(dat(zd,n)<0)= Q.l(s,zd,t);
rep(t,s,n,'0','Rev')=      sum(zd(z)$(dat(z,n)>0),  price(t,zd)*Q.l(s,z,t));

rep(t,'TOT',n,'+','prod')=sum(s,rep(t,s,n,'+','prod'));
rep(t,'TOT',n,'+','flow')=sum(s,rep(t,s,n,'+','flow'));
rep(t,'TOT',n,'-','flow')=sum(s,rep(t,s,n,'-','flow'));
rep(t,'TOT',n,'+','stor')=sum(s,rep(t,s,n,'+','stor'));
rep(t,'TOT',n,'-','stor')=sum(s,rep(t,s,n,'-','stor'));
rep(t,'TOT',n,'-',zd)=    sum(s,rep(t,s,n,'-',zd));
rep(t,'TOT',n,'price',zd)$(dat(zd,n)<0)= price(t,zd);

rep(t,'TOT',n,zd,'price')=    rep(t,zd,n,'0','price');
rep(t,'TOT',n,'inv','liq')=   sum(zl(z)$(dat(z,n)>0),D.l(z,t));
rep(t,'TOT',n,'inv','reg')=   sum(zr(z)$(dat(z,n)>0),D.l(z,t));
rep(t,'TOT',n,'inv','storI')= sum(zi(z)$(dat(z,n)>0),D.l(z,t));
rep(t,'TOT',n,'inv','storX')= sum(zx(z)$(dat(z,n)>0),D.l(z,t));
rep(t,'TOT',n,'inv','storW')= sum(zw(z)$(dat(z,n)>0),D.l(z,t));

Execute_unload 'results', rep;
